#include "sparse.h"

/*
./carop graph.bin n1 n2 n3 n4...
Computes the carryover opinion of node n1 to all nodes and write the output in a file named n1.
Do the same for n2, n3 and n4...
*/


int main(int argc,char** argv){
	struct sparse *mat;
	double *deg, *one, *x1, *x2, *x3;
	double minx, minx1, dx, maxdx;
	unsigned n, i, a, node;
	unsigned int r;
	FILE *file;

	mat=file2sparse(argv[1]);
	n=mat->dim;


	deg=calloc(n,sizeof(double));
	if (mat->weight==NULL) {
		for (i=0;i<n;i++) {
			deg[i]=mat->degcum[i+1]-mat->degcum[i];
		}
	}
	else {
		one=calloc(n,sizeof(double));
		for (i=0;i<n;i++) {
			one[i]=1;
		}
		prodvect(mat,one,deg);
		free(one);

		for (i=0;i<n;i++) {
			if (deg[i]==0) {
				deg[i]=1;
			}
		}
	}



	x1=calloc(n,sizeof(double));
	x2=calloc(n,sizeof(double));

	for (a=2;a<argc;a++) {
		
		node=atoi(argv[a]);
		printf("Computing carryover opinion from node %u:\n",node);
		x1[node]=1.,r=0;
		do {

			maxdx=0., minx=1;
			
			/* step 1: averaging */
			prodvect(mat,x1,x2);
			for (i=0;i<n;i++) {
				x2[i]/=deg[i];
				minx=((minx>x2[i]) && (x2[i]>0.)) ? x2[i]:minx;
			}
			/* step 2: resetting */
			x2[node]=1.;

			/* step 3: rescaling */
			minx1=1-minx;
			for (i=0;i<n;i++) {
				if (x2[i]>0){
				x2[i]=(x2[i]-minx)/minx1;
				dx=x2[i]-x1[i],	dx=(dx>-dx) ? dx : -dx, maxdx = (maxdx>dx) ? maxdx : dx;
				}
			}

			x3=x1,x1=x2,x2=x3;
			if (++r%10==0) {
				printf("iteration: %u, error: %.2le\n",r,maxdx);
			}
		} while (maxdx>EPS && r<REP);

		printf("iteration: %u, error: %.2le\n",r,maxdx);

		printf("Writing the result for node %u\n",node);
		file=fopen(argv[a],"w");
		for (i=0;i<n;i++) {
			fprintf(file,"%u %le\n",i,x1[i]);
		}
		fclose(file);
		
	}
	return 0;
}


