#include "sparse.h"

/*

Usage: ./heat graph.bin input output

See the nodes as iron balls and edges as iron bars, the whole graph in vacuum.
Fixes the temperatures of some nodes as described in the file input.
Computes the temperature of all nodes at equilibrium and writes the result in the file output.

*/


int main(int argc,char** argv){
	struct sparse *mat, *lap, *lap2;
	unsigned n,nl,i,j,n2;
	unsigned *lines;
	double *values, *vect, *vect2, *x, *y;
	char *output;
	FILE *file;

	if (argc==3)
		output="output";
	else
		output=argv[3];

	mat=file2sparse(argv[1]);	
	
	n=mat->dim;

	lap=mklap(mat);
	freesparse(mat);

	printf("Reading input from file %s\n",argv[2]);
	file=fopen(argv[2],"r");
	fscanf(file,"%u",&nl);
	n2=n-nl;
	lines=calloc(nl,sizeof(unsigned));
	values=calloc(nl,sizeof(double));
	for (i=0;i<nl;i++) {
		fscanf(file,"%u %lf",&lines[i],&values[i]);
	}
	fclose(file);

	printf("building boundary conditions\n");

	vect=calloc(n,sizeof(double));
	for (i=0;i<nl;i++) {
		vect[lines[i]]=values[i];
	}
	vect2=calloc(n,sizeof(double));
	
	prodvect(lap,vect,vect2);

	free(vect);

	vect=calloc(n2,sizeof(double));
	j=0;
	for (i=0;i<n;i++) {
		if ((j<nl) && (i==lines[j])) {
			j++;
			continue;
		}
		vect[i-j]=-vect2[i];
	}
	free(vect2);

	lap2=rmlines(lap,lines,nl);

	freesparse(lap);

	y=conjgrad(lap2,vect);
	freesparse(lap2),free(vect);
	
	printf("Writing the result to file %s\n",output);
	x=calloc(n,sizeof(double));
	j=0;
	for (i=0;i<n;i++) {
		if ((j<nl) && (i==lines[j])) {
			j++;
			continue;
		}
		x[i]=y[i-j];
	}
	free(y);


	for (i=0;i<nl;i++) {
		x[lines[i]]=values[i];
	}
	free(lines),free(values);

	file=fopen(output,"w");
	for (i=0;i<n;i++) {
		fprintf(file,"%u %lf\n",i,x[i]);
	}
	free(x);
	fclose(file);

	return 0;
}


