#include "sparse.h"
#include <time.h> 

void freesparse(struct sparse *mat){
	if (mat->weight!=NULL)
		free(mat->weight);
	free(mat->degcum),free(mat->neigh),free(mat);
}

struct sparse *file2sparse(char* file_name) {
	struct sparse *mat=malloc(sizeof(struct sparse));
	unsigned n,e;
	double t;
	FILE *file;
	
	//printf("Charging graph from file %s\n",file_name);

	file=fopen(file_name,"r");
	fread(&(mat->dim),sizeof(unsigned),1,file);
	n=mat->dim;
	mat->degcum=calloc(n+1,sizeof(unsigned));
	fread(mat->degcum,sizeof(unsigned),n+1,file);
	e=mat->degcum[n];
	mat->neigh=calloc(e,sizeof(unsigned));
	fread(mat->neigh,sizeof(unsigned),e,file);

	fread(&t,sizeof(double),1,file);
	if (feof(file)!=0) {
		mat->weight=NULL; 
		//printf("Unweighted graph charged\n");
	}
	else {
		mat->weight=calloc(e,sizeof(double));
		mat->weight[0]=t;
		fread(mat->weight+1,sizeof(double),e-1,file);
		printf("Weighted graph charged\n");
	}
	fclose(file);
	return mat;
}

void sparse2file(struct sparse *mat,char* file_name) {
	unsigned n=mat->dim,e=mat->degcum[n];
	FILE *file;

	file=fopen(file_name,"w");
	if (mat->weight==NULL)
		printf("Writing unweighted graph to file %s\n",file_name);
	else
		printf("Writing weighted graph to file %s\n",file_name);

	fwrite(&n,sizeof(unsigned),1,file);
	fwrite(mat->degcum,sizeof(unsigned),n+1,file);
	fwrite(mat->neigh,sizeof(unsigned),e,file);

	if (mat->weight!=NULL)
		fwrite(mat->weight,sizeof(double),e,file);

	fclose(file);
	printf("Graph written\n");
}


/*Build the laplacian of G*/
struct sparse *mklap(struct sparse *mat) {
	struct sparse *lap;
	unsigned i,j,n=mat->dim;
	double w;
	
	lap=malloc(sizeof(struct sparse));
	lap->dim=n;
	lap->degcum=calloc(n+1,sizeof(unsigned));
	lap->neigh=calloc(mat->degcum[n]+n,sizeof(unsigned));
	lap->weight=calloc(mat->degcum[n]+n,sizeof(double));

	if (mat->weight==NULL) {
		printf("Building laplacian of the unweighted graph\n");
		for (i=0;i<n;i++) {
			w=0;
			for (j=mat->degcum[i];j<mat->degcum[i+1];j++){
				lap->neigh[j+i]=mat->neigh[j];
				lap->weight[j+i]=-1.;
				w+=1.;
			}
			
			lap->neigh[j+i]=i;
			lap->weight[j+i]=w;
			lap->degcum[i+1]=mat->degcum[i+1]+i+1;
		}
	}

	else {
		printf("Building laplacian of the weighted graph\n");
		for (i=0;i<n;i++) {
			w=0;
			for (j=mat->degcum[i];j<mat->degcum[i+1];j++){
				lap->neigh[j+i]=mat->neigh[j];
				lap->weight[j+i]=-mat->weight[j];
				w+=mat->weight[j];
			}
			
			lap->neigh[j+i]=i;
			lap->weight[j+i]=w;
			lap->degcum[i+1]=mat->degcum[i+1]+i+1;
		}
	}
	printf("Laplacian built\n");
	return lap;
}

/*Remove the nl lines and colums listed in lines from the sparse matrix A*/
struct sparse *rmlines(struct sparse *mat,unsigned *lines,unsigned nl) {
	struct sparse *mat2;
	unsigned i,j,il=0,jl,n=mat->dim,e=mat->degcum[n],e2=e,d,k=0,m;
	unsigned *neigh2=calloc(e,sizeof(unsigned));

	for (i=0;i<e;i++) {
		neigh2[i]=mat->neigh[i];
	}
	
	mat2=malloc(sizeof(struct sparse));
	mat2->dim=n-nl;
	mat2->degcum=calloc(n-nl+1,sizeof(unsigned));
	for (i=0;i<n;i++) {
		if ((il<nl)&&(i==lines[il])) {
			e2-=mat->degcum[i+1]-mat->degcum[i];
			for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
				neigh2[j]=n;
			}
			il++;
			continue;
		}
		k++;
		d=0;
		for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
			m=1;
			for (jl=0;jl<nl;jl++) {
				if (neigh2[j]==lines[jl]) {
					e2--, neigh2[j]=n, d++, m=0;
					break;
					}
				if (neigh2[j]<lines[jl]) {
					neigh2[j]-=jl, m=0;
					break;
					}
			}
			if (m) {
				neigh2[j]-=nl;
;			}
		}
		mat2->degcum[k]=mat2->degcum[k-1]+mat->degcum[i+1]-mat->degcum[i]-d;
	}
	mat2->neigh=calloc(e2,sizeof(unsigned));

	if (mat->weight==NULL) {
		mat2->weight=NULL;
		j=0;
		for (i=0;i<e;i++) {
			if (neigh2[i]!=n) {
				mat2->neigh[j]=neigh2[i];
				j++;
			}
		}
	}

	else {
		mat2->weight=calloc(e2,sizeof(double));
		j=0;
		for (i=0;i<e;i++) {
			if (neigh2[i]!=n) {
				mat2->neigh[j]=neigh2[i];
				mat2->weight[j]=mat->weight[i];
				j++;
			}
		}
	}
	
	free(neigh2);
	
	return mat2;
}

/*compute the vector matvect, product of the matrix mat and vector vect*/
void prodvect(struct sparse *mat,double *vect,double *matvect) {
	unsigned n,i,j;
	n=mat->dim;
	if (mat->weight==NULL) {
		for (i=0;i<n;i++) {
			matvect[i]=0;
			for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
				matvect[i]+=vect[mat->neigh[j]];
			}
		}
	}
	else {
		for (i=0;i<n;i++) {
			matvect[i]=0;
			for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
				matvect[i]+=vect[mat->neigh[j]]*mat->weight[j];
			}
		}
	}
}


double **vp(struct sparse *mat,unsigned d) {
	int i,j,k,n=mat->dim;
	double **x=calloc(d,sizeof(double*));
	double *x_old=calloc(n,sizeof(double)),*x_new=calloc(n,sizeof(double)),*kx=calloc(n,sizeof(double)),*ix=calloc(n,sizeof(double)),*kix=calloc(n,sizeof(double));
	double s,eps=1,v,temp,ikx,ikix;

	srand(time(NULL));

	for (i=0;i<d;i++){
		s=0;

		for (j=0;j<n;j++){
			x_old[j]=rand();
			s+=x_old[j]*x_old[j];
		}
		s=sqrt(s);
		for (j=0;j<n;j++){
			x_old[j]/=s;
		}

		do{

			ikx=0;
			ikix=0;
			temp=0;
			for(j=0;j<n;j++){
				kx[j]=0;
				ix[j]=0;
				kix[j]=0;
			}

			prodvect(mat,x_old,kx);
			
			for(j=0;j<n;j++){
				ikx+=kx[j];
				temp+=x_old[j];
			}
			ikx/=n;
			temp/=n;

			for(j=0;j<n;j++){
				ix[j]=temp;
			}
			prodvect(mat,ix,kix);
			for(j=0;j<n;j++){
				ikix+=kix[j];
			}
			ikix/=n;
			
			for(j=0;j<n;j++){
				x_new[j]=kx[j]-ikx-kix[j]+ikix;
			}

			for (j=0;j<i;j++){
				s=0;
				for (k=0;k<n;k++){
					s+=x_new[k]*x[j][k];
				}
				for (k=0;k<n;k++){
					x_new[k]-=s*x[j][k];
				}
			}

			s=0;
			for (j=0;j<n;j++){
				s+=x_new[j]*x_new[j];
			}
	
			s=sqrt(s);
			eps=0;
			for (j=0;j<n;j++){
				if (j==0){//
					v=x_new[j]/x_old[j];//
				}//
				x_new[j]=x_new[j]/s;
				eps+=(x_new[j]-x_old[j])*(x_new[j]-x_old[j]);
				x_old[j]=x_new[j];
				x_new[j]=0;
			}

		}while (eps>EPS);
		printf("%lf\n",v);//
		

		x[i]=calloc(n,sizeof(double));
		
		for (j=0;j<n;j++){
			x[i][j]=x_old[j];
		}

	}
	free(x_new),free(x_old),free(kx),free(ix),free(kix);
	return x;

}


double getij(struct sparse *mat,unsigned i,unsigned j) {
	unsigned k;
	for (k=mat->degcum[i];k<mat->degcum[i+1];k++){
		if (mat->neigh[k]==j) {
			if (mat->weight==NULL)
				return 1.;
			else
				return mat->weight[k];
		}
	}
	return 0.;
}

double *getdiag(struct sparse *mat) {
	unsigned n=mat->dim,i;
	double *diag=calloc(n,sizeof(double));
	for (i=0;i<n;i++) {
		diag[i]=getij(mat,i,i);
	}
	return diag;
}

double *getsqrtdiag(struct sparse *mat) {
	unsigned n=mat->dim,i;
	double *sqrtdiag=calloc(n,sizeof(double));
	for (i=0;i<n;i++) {
		sqrtdiag[i]=sqrt(getij(mat,i,i));
	}
	return sqrtdiag;
}

double *precond(struct sparse *mat,double *vect) {
	double *sqrtdiag=getsqrtdiag(mat);
	unsigned i,j;
	for (i=0;i<mat->dim;i++) {
		for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
			mat->weight[j]/=sqrtdiag[i]*sqrtdiag[mat->neigh[j]];
		}
		vect[i]/=sqrtdiag[i];
	}
	return sqrtdiag;
}

/*iterative preconditioned conjugate gradient method*/
double *conjgrad(struct sparse *mat,double *vect){
	double *sqrtdiag,*x,*r,*p,*ap;
	double rr1,rr2,alpha,beta,t;
	unsigned n=mat->dim,i,rep=0;
	
	printf("processing iterative preconditioned conjugate gradient method:\n");
	
	sqrtdiag=precond(mat,vect);

	rr1=0;
	r=calloc(n,sizeof(double));
	p=calloc(n,sizeof(double));
	for (i=0;i<n;i++) {
		r[i]=vect[i];
		p[i]=r[i];
		rr1+=r[i]*r[i];
	}
	
	x=calloc(n,sizeof(double));
	ap=calloc(n,sizeof(double));
	
	do {
		t=0;
		prodvect(mat,p,ap);
		for (i=0;i<n;i++) {
			t+=p[i]*ap[i];
		}
		alpha=rr1/t;
		
		rr2=0;
		for (i=0;i<n;i++) {
			x[i]+=alpha*p[i];
			r[i]-=alpha*ap[i];
			rr2+=r[i]*r[i];
		}
		
		beta=rr2/rr1,rr1=rr2;
		
		for (i=0;i<n;i++) {
			p[i]=r[i]+beta*p[i];
		}
		
		if (++rep%10==0) {
			printf("iteration: %u, error: %le\n",rep,rr2);
		}
		
	} while (rr2>ERR);
	
	printf("iteration: %u, error: %le\n",rep,rr2);
	
	for (i=0;i<n;i++) {
		x[i]/=sqrtdiag[i];
	}
	free(sqrtdiag),free(r),free(p),free(ap);
	return x;
}

/*An implementation of kmeans efficient for sparse vectors*/
unsigned *sparse_kmeans(struct sparse *mat,unsigned k) {
	unsigned *label,*nvect;
	unsigned i,j,n=mat->dim,rep;
	double *d0,*d1,**d2,**centroid;
	double d,distortion_new,distortion_old=0,eps=1;
	
	printf("processing sparseKmeans:\n");
	/*Computing the norm of each row of the matrix A (used later to find the nearest centroid)*/
	d0=calloc(n,sizeof(double));
	if (mat->weight==NULL) {
		for (i=0;i<n;i++) {
			for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
				d0[i]+=1.;
			}
		}
	}
	else {
		for (i=0;i<n;i++) {
			for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
				d0[i]+=mat->weight[j]*mat->weight[j];
			}
		}
	}
	/*initializing to random labels*/
	label=calloc(n,sizeof(unsigned));
	srand(time(NULL));
	for (i=0;i<n;i++){
		label[i]=rand()%k;
	}
	
	/*allocating memory for later used arrays*/
	centroid=calloc(k,sizeof(double*));
	for (i=0;i<k;i++){
		centroid[i]=calloc(n,sizeof(double));
	}
	
	nvect=calloc(k,sizeof(unsigned));
	
	d1=calloc(k,sizeof(double));
	
	d2=calloc(k,sizeof(double*));
	
	for (i=0;i<k;i++){
		d2[i]=calloc(n,sizeof(double));
	}
	
	rep=0;


	do {
		/*Computing centroids*/
		if (mat->weight==NULL) {
			for (i=0;i<n;i++) {
				nvect[label[i]]++;
				for (j=mat->degcum[i];j<mat->degcum[i+1];j++){
					centroid[label[i]][mat->neigh[j]]+=1.;
				}
			}
		}
		else {
			for (i=0;i<n;i++) {
				nvect[label[i]]++;
				for (j=mat->degcum[i];j<mat->degcum[i+1];j++){
					centroid[label[i]][mat->neigh[j]]+=mat->weight[j];
				}
			}
		}

		for (i=0;i<k;i++) {
			for (j=0;j<n;j++) {
				centroid[i][j]/=nvect[i];
			}
		}

		/*Computing new labels*/
		for (i=0;i<k;i++){
			prodvect(mat,centroid[i],d2[i]);
			d1[i]=0;
			for (j=0;j<n;j++){
				d1[i]+=centroid[i][j]*centroid[i][j];
				centroid[i][j]=0;
			}
			
			nvect[i]=0;
		}

		distortion_new=0;
		for (j=0;j<n;j++) {
			d=INFINITY;
			for (i=0;i<k;i++){
				if (d>d0[j]+d1[i]-2*d2[i][j]) {
					d=d0[j]+d1[i]-2*d2[i][j];
					label[j]=i;
				}
			}
			distortion_new+=d;//sqrt(d);
		}

		if (rep++==0) {
			
			printf("iteration: %u, distortion: %le\n",rep,distortion_new);
		}
		else {
			eps=distortion_old-distortion_new;
			printf("iteration: %u, distortion: %.10le, error: %.2le\n",rep,distortion_new,eps);
		}
		distortion_old=distortion_new;
	} while (eps>0);



	free(d0),free(d1),free(d2),free(centroid),free(nvect);
	return label;
}		



/*An implementation of SBM-1 efficient for sparse unweighted graphs*/
unsigned *sparse_SBM(struct sparse *mat,unsigned k) {
	unsigned *label,*nvect;
	unsigned i,j,n=mat->dim,rep;
	double *d1,**d2,**centroid,**centroidlog1,**centroidlog2,**mat_prob;
	double d,proba_new,proba_old=0,eps=1;
	
	printf("processing sparseSBM:\n");

	/*initializing to random labels*/
	label=calloc(n,sizeof(unsigned));
	srand(time(NULL));
	for (i=0;i<n;i++){
		label[i]=rand()%k;
	}
	
	/*allocating memory for later used arrays*/
	centroid=calloc(k,sizeof(double*));
	for (i=0;i<k;i++){
		centroid[i]=calloc(n,sizeof(double));
	}
	centroidlog1=calloc(k,sizeof(double*));
	for (i=0;i<k;i++){
		centroidlog1[i]=calloc(n,sizeof(double));
	}
	centroidlog2=calloc(k,sizeof(double*));
	for (i=0;i<k;i++){
		centroidlog2[i]=calloc(n,sizeof(double));
	}


	nvect=calloc(k,sizeof(unsigned));
	
	d1=calloc(k,sizeof(double));
	
	d2=calloc(k,sizeof(double*));
	
	for (i=0;i<k;i++){
		d2[i]=calloc(n,sizeof(double));
	}
	
	rep=0;


	do {
		/*Computing centroids*/
		for (i=0;i<n;i++) {
			nvect[label[i]]++;
			for (j=mat->degcum[i];j<mat->degcum[i+1];j++){
				centroid[label[i]][mat->neigh[j]]+=1.;
			}
		}

		for (i=0;i<k;i++) {
			for (j=0;j<n;j++) {
				centroid[i][j]/=nvect[i];
			}
		}

		/*Computing new labels*/
		
		for (i=0;i<k;i++){
			for (j=0;j<n;j++){
				centroidlog1[i][j]=log(1-centroid[i][j]);
				centroidlog2[i][j]=log(centroid[i][j])-centroidlog1[i][j];
				centroid[i][j]=0;
			}
			nvect[i]=0;
		}

		for (i=0;i<k;i++){
			prodvect(mat,centroidlog2[i],d2[i]);
			d1[i]=0;
			for (j=0;j<n;j++){
				d1[i]+=centroidlog1[i][j];
				centroid[i][j]=0;
			}
		}

		proba_new=0;
		for (j=0;j<n;j++) {
			d=-INFINITY;
			for (i=0;i<k;i++){
				if (d<d1[i]+d2[i][j]) {
					d=d1[i]+d2[i][j];
					label[j]=i;
				}
			}
			proba_new+=d;
		}

		if (rep++==0) {
			
			printf("iteration: %u, proba: %le\n",rep,proba_new);
		}
		else {
			eps=proba_new-proba_old;
			printf("iteration: %u, proba: %.10le, error: %.2le\n",rep,proba_new,eps);
		}
		proba_old=proba_new;
	} while (eps>0);

	for (i=0;i<n;i++){
		nvect[label[i]]++;
	}

	mat_prob=calloc(k,sizeof(double*));
	for (i=0;i<k;i++){
		mat_prob[i]=calloc(k,sizeof(double));
		for (j=0;j<n;j++){
			mat_prob[i][label[j]]+=centroid[i][j];
		}
		for (j=0;j<k;j++){
			mat_prob[i][j]/=nvect[j];
		}
		for (j=0;j<n;j++){
			centroid[i][j]=mat_prob[i][label[j]];
		}
	}
			
	for (i=0;i<k;i++){
		for (j=0;j<n;j++){
			centroidlog1[i][j]=log(1-centroid[i][j]);
			centroidlog2[i][j]=log(centroid[i][j])-centroidlog1[i][j];
			centroid[i][j]=0;
		}
		nvect[i]=0;
	}

	for (i=0;i<k;i++){
		prodvect(mat,centroidlog2[i],d2[i]);
		d1[i]=0;
		for (j=0;j<n;j++){
			d1[i]+=centroidlog1[i][j];
			centroid[i][j]=0;
		}
	}

	proba_new=0;
	for (j=0;j<n;j++) {
		d=-INFINITY;
		for (i=0;i<k;i++){
			if (d<d1[i]+d2[i][j]) {
				d=d1[i]+d2[i][j];
				label[j]=i;
			}
		}
		proba_new+=d;
	}

	printf("proba: %le\n",proba_new);

	free(d1),free(d2),free(centroid),free(centroidlog1),free(centroidlog2),free(nvect),free(mat_prob);
	return label;
}		




double jaccard(unsigned *set1,unsigned n1,unsigned *set2,unsigned n2) {
	unsigned i1=0,i2=0;
	double inter=0;

	while ((i1<n1) && (i2<n2)) {
		if (set1[i1]>set2[i2]) {
			i2++;
			continue;
		}
		if (set1[i1]==set2[i2]) {
			i1++;
			i2++;
			inter+=1;
			continue;
		}
		else {
			i1++;
			continue;
		}
	}
	return inter/(n1+n2-inter);
}



int compare_unsigned_int(void const *a, void const *b){
	unsigned const *pa = a;
	unsigned const *pb = b;
	return (*pa<*pb)?-1:1;
}


void sortsparse(struct sparse *mat) {
	unsigned i,n=mat->dim;
	if (mat->weight==NULL) {
		for (i=0;i<n;i++) {
			qsort(&(mat->neigh[mat->degcum[i]]),mat->degcum[i+1]-mat->degcum[i],sizeof(unsigned),compare_unsigned_int);
		}
	}
	else {
		printf("Warning: weighted graph not sorted\n");
	}
}

/* Computes the neighbors-jaccard matrix of matrix A*/
struct sparse *mkjac(struct sparse *mat) {
	unsigned i,j,k,n=mat->dim,index;
	unsigned *dict=calloc(n,sizeof(unsigned));
	struct sparse *jac=malloc(sizeof(struct sparse));

	jac->dim=n;
	jac->degcum=calloc(n+1,sizeof(unsigned));

	if (mat->weight!=NULL) {
		printf("Warning: building the jaccard graph of a weighted graph\n");
	}

	printf("sortsparse\n");
	sortsparse(mat);
	printf("Computing the degree sequence of Jac\n");
	//Computing the degree sequence of Jac
	
	for (i=0;i<n;i++) {
		dict[i]=n;
	}
	
	for (i=0;i<n;i++) {
		jac->degcum[i+1]=jac->degcum[i];
		dict[i]=i;
		for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
			for (k=mat->degcum[mat->neigh[j]];k<mat->degcum[mat->neigh[j]+1];k++) {
				if (dict[mat->neigh[k]]!=i) {
					dict[mat->neigh[k]]=i;
					jac->degcum[i+1]++;
				}
			}
		}
	}
	for (i=0;i<n;i++) {
		dict[i]=n;
	}
	printf("jac->degcum[n]=%u\n",jac->degcum[n]);
	printf("Computing the edges and weights of Jac\n");
	/*Computing the edges and weights of Jac*/
	jac->neigh=calloc(jac->degcum[n],sizeof(unsigned));
	jac->weight=calloc(jac->degcum[n],sizeof(double));
	index=0;
	for (i=0;i<n;i++) {
		dict[i]=i;
		for (j=mat->degcum[i];j<mat->degcum[i+1];j++) {
			for (k=mat->degcum[mat->neigh[j]];k<mat->degcum[mat->neigh[j]+1];k++) {
				if (dict[mat->neigh[k]]!=i) {
					dict[mat->neigh[k]]=i;
					jac->neigh[index]=mat->neigh[k];
					jac->weight[index]=jaccard( &(mat->neigh[mat->degcum[mat->neigh[k]]]) , mat->degcum[mat->neigh[k]+1]-mat->degcum[mat->neigh[k]] , &(mat->neigh[mat->degcum[i]]) , mat->degcum[i+1]-mat->degcum[i] );
					index++;
					//printf("%i %i %i\n",i,k,index);
				}
			}
		}
	}
	free(dict);	
	return jac;
}

unsigned *bfs(struct sparse *mat,unsigned node,unsigned dmax) {
	unsigned n=mat->dim,nneigh1=1,nneigh2=0,d=0,i,j;
	unsigned *dist=calloc(n,sizeof(unsigned)),*neigh1=calloc(dmax,sizeof(unsigned)),*neigh2=calloc(dmax,sizeof(unsigned)),*neigh3=NULL;

	for (i=0;i<n;i++) {
		dist[i]=n;
	}

	neigh1[0]=node;
	
	dist[node]=d;

	while (1) {

		d++;
		for (i=0;i<nneigh1;i++) {
			
			for (j=mat->degcum[neigh1[i]];j<mat->degcum[neigh1[i]+1];j++) {
				if (dist[mat->neigh[j]]==n) {
					neigh2[nneigh2++]=mat->neigh[j];
					dist[mat->neigh[j]]=d;
				}
			}
		}

		if (nneigh2==0) {
			break;
		}

		nneigh1=nneigh2,nneigh2=0;
		neigh3=neigh2,neigh2=neigh1,neigh1=neigh3;
	}

	return dist;

}








double *htsim(struct sparse *mat,unsigned node,unsigned maxhop, unsigned ntrials) {
	unsigned n=mat->dim,pos,d,i,j;
	unsigned *npass=calloc(n,sizeof(unsigned)),*ispass=calloc(n,sizeof(unsigned));
	double *time=calloc(n,sizeof(double));

	for (i=0;i<n;i++) {
		npass[i]=0;
		ispass[i]=ntrials;
		time[i]=0;
	}

	for (i=0;i<ntrials;i++) {

		pos=node;
		npass[pos]++;
		ispass[pos]=i;
		for (j=1;j<=maxhop;j++) {
			d=mat->degcum[pos+1]-mat->degcum[pos];
			pos=mat->neigh[mat->degcum[pos]+(rand()%d)];
			if (ispass[pos]!=i) {
				ispass[pos]=i;
				time[pos]+=j;
				npass[pos]++;
			}
		}
	}

	for (i=0;i<n;i++) {
		time[i]+=(ntrials-npass[i])*(maxhop+1);
		time[i]/=ntrials;
		}

	return time;

}



unsigned long long **nonback(struct sparse *mat, unsigned node, unsigned length) {
	unsigned i,j,k,n=mat->dim;
	unsigned *deg;
	unsigned long long **xx;

	deg=calloc(n,sizeof(unsigned));
	for (i=0;i<n;i++) {
		deg[i]=mat->degcum[i+1]-mat->degcum[i]-1;
	}
	
	xx=calloc(length+1,sizeof(unsigned long long *));
	for (i=0;i<length+1;i++) {
		xx[i]=calloc(n,sizeof(unsigned long long));
		}

	printf("Computing number of nonbacktraking paths for node %u:\n",node);

	xx[0][node]=1.;

	for (k=mat->degcum[node];k<mat->degcum[node+1];k++) {
		xx[1][mat->neigh[k]]=1;
	}

	for (j=0;j<n;j++) {
		for (k=mat->degcum[j];k<mat->degcum[j+1];k++) {
			xx[2][j]+=xx[1][mat->neigh[k]];
		}
	}
	xx[2][node]-=deg[node]+1;

	for (i=2;i<length;i++) {

		for (j=0;j<n;j++) {
			for (k=mat->degcum[j];k<mat->degcum[j+1];k++) {
				xx[i+1][j]+=xx[i][mat->neigh[k]];
			}
			xx[i+1][j]-=deg[j]*xx[i-1][j];
		}
	}

	return xx;

}






