#include "sparse.h"

#define IT 20

/*

./walkparamB graph.bin n alpha beta
computes the degree normalized pagerank with restart probability alpha to node n and normalization beta for all nodes in the graph and writes the result in a file named n_alpha_beta

Tong, H., & Faloutsos, C. (2006, August). Center-piece subgraphs: problem definition and fast solutions. In Proceedings of the 12th ACM SIGKDD international conference on Knowledge discovery and data mining (pp. 404-413). ACM.

*/


int main(int argc,char** argv){
	struct sparse *mat;
	double *deg, *x1, *x2, *x3, *norm, alpha=atof(argv[3]), beta=atof(argv[4]);
	unsigned n, i, j, a, node=atoi(argv[2]);
	FILE *file;
	char fn [100];

	printf("node,alpha,beta=%u,%lf,%lf\n",node,alpha,beta);
	mat=file2sparse(argv[1]);
	n=mat->dim;

	deg=calloc(n,sizeof(double));
	for (i=0;i<n;i++) {
		deg[i]=mat->degcum[i+1]-mat->degcum[i];
	}
	
	norm=calloc(n,sizeof(double));
	for (i=0;i<n;i++) {
		for (j=mat->degcum[i];j<mat->degcum[i+1];j++){
			norm[i]+=pow( (1./deg[mat->neigh[j]]), beta );
		}
	}


	mat->weight=calloc(mat->degcum[n],sizeof(double));
	for (i=0;i<n;i++) {
		for (j=mat->degcum[i];j<mat->degcum[i+1];j++){
			mat->weight[j]=(1-alpha)*pow( 1./deg[i], beta )/norm[mat->neigh[j]];
		}
	}

	x1=calloc(n,sizeof(double));
	x2=calloc(n,sizeof(double));


//	printf("Computing ppr for node %u:\n",node);

	x1[node]=1.;
	for (i=1;i<IT;i++) {
		//printf("%i\n",i);
		prodvect(mat,x1,x2);
		x2[node]+=alpha;
		x3=x1; x1=x2; x2=x3;
	}
	//printf("Writing the result for node %u\n",node);


	snprintf (fn, sizeof fn, "%u_%lf_%lf", node,alpha,beta);
	file=fopen(fn,"w");
	for (i=0;i<n;i++) {
		fprintf(file,"%u %le\n",i,x1[i]);
		x1[i]=0;
	}
	fclose(file);

	return 0;

}


